package dev.pfaff.altargui.texture;

import dev.pfaff.altargui.resource.LoadableResource;
import org.jspecify.annotations.NonNull;

import java.io.IOException;
import java.io.InputStream;
import java.io.UncheckedIOException;
import java.lang.foreign.MemorySegment;
import java.net.URI;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.concurrent.CancellationException;

public record UrlLoadableResource(UrlResourceLoader loader, String url) implements LoadableResource {
	@Override
	public @NonNull MemorySegment readBytes() throws IOException {
		var bytes = doReqRes(responseInfo -> {
			checkResponseInfo(responseInfo);
			return HttpResponse.BodySubscribers.ofByteArray();
		});
		return MemorySegment.ofArray(bytes);
	}

	@Override
	public @NonNull InputStream openInputStream() throws IOException {
		return doReqRes(responseInfo -> {
			checkResponseInfo(responseInfo);
			return HttpResponse.BodySubscribers.ofInputStream();
		});
	}

	/**
	 * @throws IOException if an I/O error occurs
	 * @throws CancellationException if the operation is interrupted
	 */
	private <T> T doReqRes(HttpResponse.BodyHandler<T> bodyHandler) throws IOException, CancellationException {
		var req = HttpRequest.newBuilder().GET().uri(URI.create(this.url)).build();
		try {
			return this.loader.client.send(req, bodyHandler).body();
		} catch (InterruptedException e) {
			throw new CancellationException(e.getMessage());
		}
	}

	private static void checkResponseInfo(HttpResponse.ResponseInfo responseInfo) {
		if (responseInfo.statusCode() != 200) {
			throw new UncheckedIOException(new IOException(STR."Non-OK status code \{responseInfo.statusCode()}"));
		}
	}
}
